@@ -1,10 +1,12 @@
-use std::fmt;
-
-use serde::ser::{
-    SerializeMap, SerializeSeq, SerializeStruct, SerializeStructVariant, SerializeTuple,
-    SerializeTupleStruct, SerializeTupleVariant,
+use serde::de::{
+    Deserialize, DeserializeSeed, Deserializer, EnumAccess, Error, MapAccess, SeqAccess,
+    VariantAccess, Visitor,
 };
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
+use serde::ser::{
+    Serialize, SerializeMap, SerializeSeq, SerializeStruct, SerializeStructVariant, SerializeTuple,
+    SerializeTupleStruct, SerializeTupleVariant, Serializer,
+};
+use std::fmt::{self, Display};
 
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub struct Readable<T: ?Sized>(T);
@@ -14,7 +16,7 @@ pub struct Compact<T: ?Sized>(T);
 /// Trait to determine whether a value is represented in human-readable or
 /// compact form.
 ///
-/// ```edition2021
+/// ```
 /// use serde::{Deserialize, Deserializer, Serialize, Serializer};
 /// use serde_test::{assert_tokens, Configure, Token};
 ///
@@ -84,11 +86,11 @@ pub trait Configure {
     }
 }
 
-impl<T: ?Sized> Configure for T {}
+impl<T> Configure for T where T: ?Sized {}
 
-impl<T: ?Sized> Serialize for Readable<T>
+impl<T> Serialize for Readable<T>
 where
-    T: Serialize,
+    T: ?Sized + Serialize,
 {
     #[inline]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
@@ -98,9 +100,10 @@ where
         self.0.serialize(Readable(serializer))
     }
 }
-impl<T: ?Sized> Serialize for Compact<T>
+
+impl<T> Serialize for Compact<T>
 where
-    T: Serialize,
+    T: ?Sized + Serialize,
 {
     #[inline]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
@@ -110,6 +113,7 @@ where
         self.0.serialize(Compact(serializer))
     }
 }
+
 impl<'de, T> Deserialize<'de> for Readable<T>
 where
     T: Deserialize<'de>,
@@ -120,7 +124,15 @@ where
     {
         T::deserialize(Readable(deserializer)).map(Readable)
     }
+
+    fn deserialize_in_place<D>(deserializer: D, place: &mut Self) -> Result<(), D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        T::deserialize_in_place(Readable(deserializer), &mut place.0)
+    }
 }
+
 impl<'de, T> Deserialize<'de> for Compact<T>
 where
     T: Deserialize<'de>,
@@ -131,6 +143,13 @@ where
     {
         T::deserialize(Compact(deserializer)).map(Compact)
     }
+
+    fn deserialize_in_place<D>(deserializer: D, place: &mut Self) -> Result<(), D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        T::deserialize_in_place(Compact(deserializer), &mut place.0)
+    }
 }
 
 impl<'de, T> DeserializeSeed<'de> for Readable<T>
@@ -146,6 +165,7 @@ where
         self.0.deserialize(Readable(deserializer))
     }
 }
+
 impl<'de, T> DeserializeSeed<'de> for Compact<T>
 where
     T: DeserializeSeed<'de>,
@@ -203,17 +223,18 @@ macro_rules! impl_serializer {
                 serialize_i16 i16,
                 serialize_i32 i32,
                 serialize_i64 i64,
+                serialize_i128 i128,
                 serialize_u8 u8,
                 serialize_u16 u16,
                 serialize_u32 u32,
                 serialize_u64 u64,
+                serialize_u128 u128,
                 serialize_f32 f32,
                 serialize_f64 f64,
                 serialize_char char,
                 serialize_str &str,
                 serialize_bytes &[u8],
                 serialize_unit_struct &'static str
-
             }
 
             fn serialize_unit(self) -> Result<S::Ok, S::Error> {
@@ -229,18 +250,18 @@ macro_rules! impl_serializer {
                 self.0.serialize_unit_variant(name, variant_index, variant)
             }
 
-            fn serialize_newtype_struct<T: ?Sized>(
+            fn serialize_newtype_struct<T>(
                 self,
                 name: &'static str,
                 value: &T,
             ) -> Result<S::Ok, S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_newtype_struct(name, &$wrapper(value))
             }
 
-            fn serialize_newtype_variant<T: ?Sized>(
+            fn serialize_newtype_variant<T>(
                 self,
                 name: &'static str,
                 variant_index: u32,
@@ -248,7 +269,7 @@ macro_rules! impl_serializer {
                 value: &T,
             ) -> Result<S::Ok, S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0
                     .serialize_newtype_variant(name, variant_index, variant, &$wrapper(value))
@@ -258,9 +279,9 @@ macro_rules! impl_serializer {
                 self.0.serialize_none()
             }
 
-            fn serialize_some<T: ?Sized>(self, value: &T) -> Result<S::Ok, Self::Error>
+            fn serialize_some<T>(self, value: &T) -> Result<S::Ok, Self::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_some(&$wrapper(value))
             }
@@ -316,6 +337,34 @@ macro_rules! impl_serializer {
                     .serialize_struct_variant(name, variant_index, variant, len)
                     .map($wrapper)
             }
+
+            fn collect_seq<I>(self, iter: I) -> Result<Self::Ok, Self::Error>
+            where
+                I: IntoIterator,
+                <I as IntoIterator>::Item: Serialize,
+            {
+                self.0
+                    .collect_seq(iter.into_iter().map(|item| $wrapper(item)))
+            }
+
+            fn collect_map<K, V, I>(self, iter: I) -> Result<Self::Ok, Self::Error>
+            where
+                K: Serialize,
+                V: Serialize,
+                I: IntoIterator<Item = (K, V)>,
+            {
+                self.0.collect_map(
+                    iter.into_iter()
+                        .map(|(key, value)| ($wrapper(key), $wrapper(value))),
+                )
+            }
+
+            fn collect_str<T>(self, value: &T) -> Result<Self::Ok, Self::Error>
+            where
+                T: ?Sized + Display,
+            {
+                self.0.collect_str(value)
+            }
         }
 
         impl<S> SerializeSeq for $wrapper<S>
@@ -324,12 +373,14 @@ macro_rules! impl_serializer {
         {
             type Ok = S::Ok;
             type Error = S::Error;
-            fn serialize_element<T: ?Sized>(&mut self, value: &T) -> Result<(), S::Error>
+
+            fn serialize_element<T>(&mut self, value: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_element(&$wrapper(value))
             }
+
             fn end(self) -> Result<S::Ok, S::Error> {
                 self.0.end()
             }
@@ -341,12 +392,14 @@ macro_rules! impl_serializer {
         {
             type Ok = S::Ok;
             type Error = S::Error;
-            fn serialize_element<T: ?Sized>(&mut self, value: &T) -> Result<(), S::Error>
+
+            fn serialize_element<T>(&mut self, value: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_element(&$wrapper(value))
             }
+
             fn end(self) -> Result<S::Ok, S::Error> {
                 self.0.end()
             }
@@ -358,12 +411,14 @@ macro_rules! impl_serializer {
         {
             type Ok = S::Ok;
             type Error = S::Error;
-            fn serialize_field<T: ?Sized>(&mut self, value: &T) -> Result<(), S::Error>
+
+            fn serialize_field<T>(&mut self, value: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_field(&$wrapper(value))
             }
+
             fn end(self) -> Result<S::Ok, S::Error> {
                 self.0.end()
             }
@@ -375,12 +430,14 @@ macro_rules! impl_serializer {
         {
             type Ok = S::Ok;
             type Error = S::Error;
-            fn serialize_field<T: ?Sized>(&mut self, value: &T) -> Result<(), S::Error>
+
+            fn serialize_field<T>(&mut self, value: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_field(&$wrapper(value))
             }
+
             fn end(self) -> Result<S::Ok, S::Error> {
                 self.0.end()
             }
@@ -392,29 +449,29 @@ macro_rules! impl_serializer {
         {
             type Ok = S::Ok;
             type Error = S::Error;
-            fn serialize_key<T: ?Sized>(&mut self, key: &T) -> Result<(), S::Error>
+
+            fn serialize_key<T>(&mut self, key: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_key(&$wrapper(key))
             }
-            fn serialize_value<T: ?Sized>(&mut self, value: &T) -> Result<(), S::Error>
+
+            fn serialize_value<T>(&mut self, value: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_value(&$wrapper(value))
             }
-            fn serialize_entry<K: ?Sized, V: ?Sized>(
-                &mut self,
-                key: &K,
-                value: &V,
-            ) -> Result<(), S::Error>
+
+            fn serialize_entry<K, V>(&mut self, key: &K, value: &V) -> Result<(), S::Error>
             where
-                K: Serialize,
-                V: Serialize,
+                K: ?Sized + Serialize,
+                V: ?Sized + Serialize,
             {
-                self.0.serialize_entry(key, &$wrapper(value))
+                self.0.serialize_entry(&$wrapper(key), &$wrapper(value))
             }
+
             fn end(self) -> Result<S::Ok, S::Error> {
                 self.0.end()
             }
@@ -426,16 +483,18 @@ macro_rules! impl_serializer {
         {
             type Ok = S::Ok;
             type Error = S::Error;
-            fn serialize_field<T: ?Sized>(
-                &mut self,
-                name: &'static str,
-                field: &T,
-            ) -> Result<(), S::Error>
+
+            fn serialize_field<T>(&mut self, name: &'static str, field: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_field(name, &$wrapper(field))
             }
+
+            fn skip_field(&mut self, key: &'static str) -> Result<(), Self::Error> {
+                self.0.skip_field(key)
+            }
+
             fn end(self) -> Result<S::Ok, S::Error> {
                 self.0.end()
             }
@@ -447,16 +506,18 @@ macro_rules! impl_serializer {
         {
             type Ok = S::Ok;
             type Error = S::Error;
-            fn serialize_field<T: ?Sized>(
-                &mut self,
-                name: &'static str,
-                field: &T,
-            ) -> Result<(), S::Error>
+
+            fn serialize_field<T>(&mut self, name: &'static str, field: &T) -> Result<(), S::Error>
             where
-                T: Serialize,
+                T: ?Sized + Serialize,
             {
                 self.0.serialize_field(name, &$wrapper(field))
             }
+
+            fn skip_field(&mut self, key: &'static str) -> Result<(), Self::Error> {
+                self.0.skip_field(key)
+            }
+
             fn end(self) -> Result<S::Ok, S::Error> {
                 self.0.end()
             }
@@ -467,8 +528,6 @@ macro_rules! impl_serializer {
 impl_serializer!(Readable, true);
 impl_serializer!(Compact, false);
 
-use serde::de::{DeserializeSeed, EnumAccess, Error, MapAccess, SeqAccess, VariantAccess, Visitor};
-
 macro_rules! forward_deserialize_methods {
     ( $wrapper : ident ( $( $name: ident ),* ) ) => {
         $(
@@ -498,10 +557,12 @@ macro_rules! impl_deserializer {
                     deserialize_u16,
                     deserialize_u32,
                     deserialize_u64,
+                    deserialize_u128,
                     deserialize_i8,
                     deserialize_i16,
                     deserialize_i32,
                     deserialize_i64,
+                    deserialize_i128,
                     deserialize_f32,
                     deserialize_f64,
                     deserialize_char,
@@ -528,6 +589,7 @@ macro_rules! impl_deserializer {
             {
                 self.0.deserialize_unit_struct(name, $wrapper(visitor))
             }
+
             fn deserialize_newtype_struct<V>(
                 self,
                 name: &'static str,
@@ -538,12 +600,14 @@ macro_rules! impl_deserializer {
             {
                 self.0.deserialize_newtype_struct(name, $wrapper(visitor))
             }
+
             fn deserialize_tuple<V>(self, len: usize, visitor: V) -> Result<V::Value, D::Error>
             where
                 V: Visitor<'de>,
             {
                 self.0.deserialize_tuple(len, $wrapper(visitor))
             }
+
             fn deserialize_tuple_struct<V>(
                 self,
                 name: &'static str,
@@ -556,6 +620,7 @@ macro_rules! impl_deserializer {
                 self.0
                     .deserialize_tuple_struct(name, len, $wrapper(visitor))
             }
+
             fn deserialize_struct<V>(
                 self,
                 name: &'static str,
@@ -567,6 +632,7 @@ macro_rules! impl_deserializer {
             {
                 self.0.deserialize_struct(name, fields, $wrapper(visitor))
             }
+
             fn deserialize_enum<V>(
                 self,
                 name: &'static str,
@@ -589,153 +655,193 @@ macro_rules! impl_deserializer {
             D: Visitor<'de>,
         {
             type Value = D::Value;
+
             fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
                 self.0.expecting(formatter)
             }
+
             fn visit_bool<E>(self, v: bool) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_bool(v)
             }
+
             fn visit_i8<E>(self, v: i8) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_i8(v)
             }
+
             fn visit_i16<E>(self, v: i16) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_i16(v)
             }
+
             fn visit_i32<E>(self, v: i32) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_i32(v)
             }
+
             fn visit_i64<E>(self, v: i64) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_i64(v)
             }
+
+            fn visit_i128<E>(self, v: i128) -> Result<D::Value, E>
+            where
+                E: Error,
+            {
+                self.0.visit_i128(v)
+            }
+
             fn visit_u8<E>(self, v: u8) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_u8(v)
             }
+
             fn visit_u16<E>(self, v: u16) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_u16(v)
             }
+
             fn visit_u32<E>(self, v: u32) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_u32(v)
             }
+
             fn visit_u64<E>(self, v: u64) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_u64(v)
             }
+
+            fn visit_u128<E>(self, v: u128) -> Result<D::Value, E>
+            where
+                E: Error,
+            {
+                self.0.visit_u128(v)
+            }
+
             fn visit_f32<E>(self, v: f32) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_f32(v)
             }
+
             fn visit_f64<E>(self, v: f64) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_f64(v)
             }
+
             fn visit_char<E>(self, v: char) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_char(v)
             }
+
             fn visit_str<E>(self, v: &str) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_str(v)
             }
+
             fn visit_borrowed_str<E>(self, v: &'de str) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_borrowed_str(v)
             }
+
             fn visit_string<E>(self, v: String) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_string(v)
             }
+
             fn visit_bytes<E>(self, v: &[u8]) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_bytes(v)
             }
+
             fn visit_borrowed_bytes<E>(self, v: &'de [u8]) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_borrowed_bytes(v)
             }
+
             fn visit_byte_buf<E>(self, v: Vec<u8>) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_byte_buf(v)
             }
+
             fn visit_none<E>(self) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_none()
             }
+
             fn visit_some<D2>(self, deserializer: D2) -> Result<Self::Value, D2::Error>
             where
                 D2: Deserializer<'de>,
             {
                 self.0.visit_some($wrapper(deserializer))
             }
+
             fn visit_unit<E>(self) -> Result<D::Value, E>
             where
                 E: Error,
             {
                 self.0.visit_unit()
             }
+
             fn visit_newtype_struct<D2>(self, deserializer: D2) -> Result<Self::Value, D2::Error>
             where
                 D2: Deserializer<'de>,
             {
                 self.0.visit_newtype_struct($wrapper(deserializer))
             }
+
             fn visit_seq<V>(self, seq: V) -> Result<D::Value, V::Error>
             where
                 V: SeqAccess<'de>,
             {
                 self.0.visit_seq($wrapper(seq))
             }
+
             fn visit_map<V>(self, map: V) -> Result<D::Value, V::Error>
             where
                 V: MapAccess<'de>,
             {
                 self.0.visit_map($wrapper(map))
             }
+
             fn visit_enum<V>(self, data: V) -> Result<D::Value, V::Error>
             where
                 V: EnumAccess<'de>,
@@ -749,12 +855,14 @@ macro_rules! impl_deserializer {
             D: SeqAccess<'de>,
         {
             type Error = D::Error;
+
             fn next_element_seed<T>(&mut self, seed: T) -> Result<Option<T::Value>, D::Error>
             where
                 T: DeserializeSeed<'de>,
             {
                 self.0.next_element_seed($wrapper(seed))
             }
+
             fn size_hint(&self) -> Option<usize> {
                 self.0.size_hint()
             }
@@ -765,18 +873,21 @@ macro_rules! impl_deserializer {
             D: MapAccess<'de>,
         {
             type Error = D::Error;
+
             fn next_key_seed<K>(&mut self, seed: K) -> Result<Option<K::Value>, D::Error>
             where
                 K: DeserializeSeed<'de>,
             {
                 self.0.next_key_seed($wrapper(seed))
             }
+
             fn next_value_seed<V>(&mut self, seed: V) -> Result<V::Value, D::Error>
             where
                 V: DeserializeSeed<'de>,
             {
                 self.0.next_value_seed($wrapper(seed))
             }
+
             fn next_entry_seed<K, V>(
                 &mut self,
                 kseed: K,
@@ -788,6 +899,7 @@ macro_rules! impl_deserializer {
             {
                 self.0.next_entry_seed($wrapper(kseed), $wrapper(vseed))
             }
+
             fn size_hint(&self) -> Option<usize> {
                 self.0.size_hint()
             }
@@ -799,6 +911,7 @@ macro_rules! impl_deserializer {
         {
             type Error = D::Error;
             type Variant = $wrapper<D::Variant>;
+
             fn variant_seed<V>(self, seed: V) -> Result<(V::Value, Self::Variant), Self::Error>
             where
                 V: DeserializeSeed<'de>,
@@ -814,21 +927,25 @@ macro_rules! impl_deserializer {
             D: VariantAccess<'de>,
         {
             type Error = D::Error;
+
             fn unit_variant(self) -> Result<(), D::Error> {
                 self.0.unit_variant()
             }
+
             fn newtype_variant_seed<T>(self, seed: T) -> Result<T::Value, D::Error>
             where
                 T: DeserializeSeed<'de>,
             {
                 self.0.newtype_variant_seed($wrapper(seed))
             }
+
             fn tuple_variant<V>(self, len: usize, visitor: V) -> Result<V::Value, D::Error>
             where
                 V: Visitor<'de>,
             {
                 self.0.tuple_variant(len, $wrapper(visitor))
             }
+
             fn struct_variant<V>(
                 self,
                 fields: &'static [&'static str],
